
from selenium.webdriver.common.by import By

class LoginLocators(object):
    USERNAME_INPUT = (By.NAME, 'j_username')
    PASSWORD_INPUT = (By.NAME, 'j_password')
    SUBMIT_BUTTON = (By.NAME, 'submit')

class AdminLocators(object):
    RECORDINGS_TAB = (By.CSS_SELECTOR, 'a#i18n_tab_recording')

class RecordingsLocators(object):
    UPLOAD_RECORDING_BUTTON = (By.CSS_SELECTOR, 'button#uploadButton')
    SEARCH_SELECT = (By.CSS_SELECTOR, 'div#searchBox > select')
    SEARCH_INPUT = (By.CSS_SELECTOR, 'div#searchBox > span > input')
    FILTER_FOUND_COUNT = (By.CSS_SELECTOR, 'span#filterRecordingCount')
    PERPAGE_SELECT = (By.CSS_SELECTOR, 'select#pageSize')
    REFRESH_CHECKBOX = (By.CSS_SELECTOR, 'input#refreshEnabled')
    ON_HOLD_TAB = (By.CSS_SELECTOR, 'label[for=state-hold]')
    TRIM_LINK = (By.CSS_SELECTOR, 'a[title="Review / VideoEdit"]')
    TRIM_IFRAME = (By.CSS_SELECTOR, 'iframe#holdActionUI')

class UploadLocators(object):
    TITLE_INPUT = (By.CSS_SELECTOR, 'input#title')
    PRESENTER_INPUT = (By.CSS_SELECTOR, 'input#creator')
    COURSE_INPUT = (By.CSS_SELECTOR, 'input#seriesSelect')
    LICENSE_SELECT = (By.CSS_SELECTOR, 'select#licenseField')
    REC_DATE_INPUT = (By.CSS_SELECTOR, 'input#recordDate')
    START_HOUR_SELECT = (By.CSS_SELECTOR, 'select#startTimeHour')
    START_MINUTE_SELECT = (By.CSS_SELECTOR, 'select#startTimeMin')
    CONTRIBUTOR_INPUT = (By.CSS_SELECTOR, 'input#contributor')
    TYPE_INPUT = (By.CSS_SELECTOR, 'input#type')
    SUBJECT_INPUT = (By.CSS_SELECTOR, 'input#subject')
    LANG_INPUT = (By.CSS_SELECTOR, 'input#language')
    DESC_INPUT = (By.CSS_SELECTOR, 'input#description')
    COPYRIGHT_INPUT = (By.CSS_SELECTOR, 'input#copyright')

    MULTI_FILE_RADIO = (By.CSS_SELECTOR, 'input#multiUploadRadio')
    SINGLE_FILE_RADIO = (By.CSS_SELECTOR, 'input#singleUploadRadio')

    FILE_INPUT_IFRAME = (By.CSS_SELECTOR, 'iframe.uploadForm-container')

    SINGLE_FILE_LOCAL_RADIO = (By.CSS_SELECTOR,'input#fileSourceSingleA')
    SINGLE_FILE_INBOX_RADIO = (By.CSS_SELECTOR, 'input#fileSourceSingleB')

    MULTI_FILE_PRESENTATION_LOCAL_RADIO = (By.CSS_SELECTOR, 'input#fileSourcePresentationA')
    MULTI_FILE_PRESENTATION_INBOX_RADIO = (By.CSS_SELECTOR, 'input#fileSourcePresentationB')
    MULTI_FILE_PRESENTER_LOCAL_RADIO = (By.CSS_SELECTOR, 'input#fileSourcePresenterA')
    MULTI_FILE_PRESENTER_INBOX_RADIO = (By.CSS_SELECTOR, 'input#fileSourcePresenterB')

    LOCAL_FILE_SELECTOR = (By.CSS_SELECTOR, 'input#file')
    INBOX_FILE_SELECT = (By.CSS_SELECTOR, 'select#file')

    CONTAINS_SLIDES_CHECKBOX = (By.CSS_SELECTOR, 'input#containsSlides')
    WORKFLOW_SELECT = (By.CSS_SELECTOR, 'select#workflowSelector')
    LIVE_STREAM_CHECKBOX = (By.CSS_SELECTOR, 'input#hasLive')
    MULTITRACK_CHECKBOX = (By.CSS_SELECTOR, 'input#epiphanUpload')
    UPLOAD_BUTTON = (By.CSS_SELECTOR, 'button#submitButton')
    UPLOAD_PROGRESS_DIALOG = (By.CSS_SELECTOR, 'div#progressStage')

class TrimLocators(object):
    SHORTCUT_TABLE = (By.CSS_SELECTOR, 'div#rightBox')
    CLIP_BEGIN_INPUT = (By.CSS_SELECTOR, 'span#clipBegin > input')
    CLIP_END_INPUT = (By.CSS_SELECTOR, 'span#clipEnd > input')
    CLIP_OK_BUTTON = (By.CSS_SELECTOR, 'input#okButton')
    CLIP_REMOVE_BUTTON = (By.CSS_SELECTOR, 'a#splitRemover-0')
    CONTINUE_BUTTON = (By.CSS_SELECTOR, 'input#continueButton')
    CLEAR_BUTTON = (By.CSS_SELECTOR, 'input#clearList')




#!/usr/bin/env python
from time import sleep

import click
from fabric.context_managers import cd
from selenium.common.exceptions import TimeoutException

click.disable_unicode_literals_warning = True

from unipath import Path
from fabric.api import run, abort, env, hide, sudo
from fabric.operations import put
from fabric.colors import cyan
from fabric.contrib.files import exists as remote_exists

from mh_selenium.utils import selenium_options, inbox_options, pass_state, \
    init_driver, base_url_arg, init_fabric
from mh_selenium.pages import RecordingsPage, AdminPage, TrimPage, \
                              UploadPage

@click.group()
def cli():
    pass

@cli.command()
@click.option('--presenter')
@click.option('--presentation')
@click.option('--combined')
@click.option('--title', default='mh-selenium upload')
@click.option('-i', '--inbox', is_flag=True)
@click.option('--live_stream', is_flag=True)
@selenium_options
@pass_state
@init_driver('/admin')
def upload(state, presenter, presentation, combined, title, inbox, live_stream):

    page = RecordingsPage(state.driver)
    page.upload_recording_button.click()

    page = UploadPage(state.driver)

    page.enter_text(page.title_input, title)
    page.enter_text(page.type_input, "L01")
    page.set_upload_files(presenter=presenter,
                          presentation=presentation,
                          combined=combined,
                          is_inbox=inbox)
    page.workflow_select.select_by_value('DCE-production')
    page.set_live_stream(live_stream)
    page.set_multitrack(combined is not None)
    page.upload_button.click()
    page.wait_for_upload_finish()

@cli.command()
@click.option('-f', '--filter')
@click.option('-c', '--count', type=int, default=None)
@selenium_options
@pass_state
@init_driver('/admin')
def trim(state, filter=None, count=None):

    page = AdminPage(state.driver)
    page.recordings_tab.click()
    page = RecordingsPage(state.driver)
    page.max_per_page()
    page.switch_to_tab(page.on_hold_tab)

    if filter is not None:
        field, value = filter.split(':', 1)
        page.filter_recordings(field, value)

    link_idx = 0
    while True:

        # kinda annoying that we have to do this each time
        page.refresh_off()

        try:
            # re-resolve the trim link elements each time because the refs go
            # stale when the page reloads
            # also, iterate via incrementing idx so that we don't somehow trim
            # the same thing > once (e.g. the entry doesn't get removed from
            # the table because the workflow hasn't actually resumed)
            link = page.trim_links[link_idx]
        except (TimeoutException,IndexError):
            break

        href = link.get_attribute('href')
        scheme, js = href.split(':', 1)
        page.js(js)
        page.switch_frame(page.trim_iframe)
        page = TrimPage(state.driver)
        page.trim()
        page.default_frame()
        page.reload()
        page = RecordingsPage(state.driver)

        if count is not None:
            count -= 1
            if count == 0:
                break

@cli.group()
@pass_state
def inbox(state):
    pass

@inbox.command(name='put')
@click.option('-f', '--file')
@inbox_options
@pass_state
@init_fabric
def inbox_put(state, file):
    result = put(local_path=file, remote_path=state.inbox, use_sudo=True)
    print(cyan("Files created: {}".format(str(result))))

@inbox.command(name='symlink')
@click.option('-f', '--file')
@click.option('-c', '--count', type=int, default=1)
@inbox_options
@pass_state
@init_fabric
def inbox_symlink(state, file, count):

    remote_path = state.inbox_dest.child(file)
    if not remote_exists(remote_path, verbose=True):
        abort("remote file {} not found".format(remote_path))

    with cd(state.inbox):
        for i in range(count):
            link = remote_path.stem + '_' + str(i + 1) + remote_path.ext
            sudo("ln -s {} {}".format(remote_path, link))
        return

@inbox.command(name='list')
@click.argument('match', default='')
@inbox_options
@pass_state
@init_fabric
def inbox_list(state, match):

    if not remote_exists(state.inbox_dest):
        return
    with cd(state.inbox_dest), hide('running', 'stdout', 'stderr'):
        output = run('ls {}'.format(match))
        for f in output.split():
            print(cyan(f))


if __name__ == '__main__':
    cli()



import datetime
from os.path import abspath
from time import sleep
from selenium.webdriver.support.ui import Select
from selenium.webdriver.common.keys import Keys
from pytimeparse.timeparse import timeparse

from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.expected_conditions import \
    element_to_be_clickable as clickable, \
    presence_of_element_located as present, \
    presence_of_all_elements_located as all_present, \
    staleness_of as stale, \
    visibility_of as visible, \
    text_to_be_present_in_element_value as elem_value

from locators import RecordingsLocators, \
                                             UploadLocators, \
                                             TrimLocators, \
                                             LoginLocators, \
                                             AdminLocators

class BasePage(object):
    def __init__(self, driver):
        self.driver = driver

    def reload(self):
        self.driver.refresh()

    def default_frame(self):
        self.driver.switch_to.default_content()

    def js(self, *args):
        self.driver.execute_script(*args)

    def switch_frame(self, frame_elem):
        self.driver.switch_to.frame(frame_elem)

    def get_element(self, locator, condition=clickable):
        if condition is not None:
            return WebDriverWait(self.driver, 10).until(condition(locator))
        else:
            return self.driver.find_element(*locator)

    def get_elements(self, locator, condition=all_present):
        if condition is not None:
            return WebDriverWait(self.driver, 10).until(condition(locator))
        else:
            return self.driver.find_elements(*locator)

    def set_checkbox(self, cb, enabled):
        if enabled and not cb.is_selected():
            cb.click()
        elif not enabled and cb.is_selected():
            cb.click()

    def enter_text(self, elem, text):
        elem.send_keys(text)
        return WebDriverWait(self.driver, 10).until(
            lambda x: elem.get_attribute('value') == text
        )

class LoginPage(BasePage):

    @property
    def username_input(self):
        return self.get_element(LoginLocators.USERNAME_INPUT)

    @property
    def password_input(self):
        return self.get_element(LoginLocators.PASSWORD_INPUT)

    @property
    def submit(self):
        return self.get_element(LoginLocators.SUBMIT_BUTTON)

    def login(self, username, password):
        self.username_input.send_keys(username)
        self.password_input.send_keys(password)
        self.submit.click()

class AdminPage(BasePage):

    @property
    def recordings_tab(self):
        return self.get_element(AdminLocators.RECORDINGS_TAB)

class RecordingsPage(AdminPage):

    @property
    def upload_recording_button(self):
        return self.get_element(RecordingsLocators.UPLOAD_RECORDING_BUTTON)

    @property
    def search_select(self):
        return Select(self.get_element(RecordingsLocators.SEARCH_SELECT, clickable))

    @property
    def search_input(self):
        return self.get_element(RecordingsLocators.SEARCH_INPUT, clickable)

    @property
    def per_page_select(self):
        return Select(self.get_element(RecordingsLocators.PERPAGE_SELECT, clickable))

    @property
    def refresh_checkbox(self):
        return self.get_element(RecordingsLocators.REFRESH_CHECKBOX, clickable)

    @property
    def on_hold_tab(self):
        return self.get_element(RecordingsLocators.ON_HOLD_TAB, clickable)


    @property
    def trim_iframe(self):
        return self.get_element(RecordingsLocators.TRIM_IFRAME)

    @property
    def trim_links(self):
        return self.get_elements(RecordingsLocators.TRIM_LINK)

    def refresh_off(self):
        self.set_checkbox(self.refresh_checkbox, False)
        self.js('window.clearInterval(ocRecordings.refreshInterval);')

    def filter_recordings(self, field, value):
        self.search_select.select_by_value(field)
        self.search_input.send_keys(value)
        self.search_input.send_keys(Keys.RETURN)
        found = self.get_element(RecordingsLocators.FILTER_FOUND_COUNT, present)
        return found

    def max_per_page(self):
        self.per_page_select.select_by_visible_text('100')

    def switch_to_tab(self, tab_elem):
        """
        bypass the usual element click() method as these tab links frequently
        throw exceptions about not being clickable at point blah, blah
        """
        self.driver.execute_script("arguments[0].click();", tab_elem)

class UploadPage(BasePage):

    @property
    def title_input(self):
        return self.get_element(UploadLocators.TITLE_INPUT)

    @property
    def presenter_input(self):
        return self.get_element(UploadLocators.PRESENTER_INPUT)

    @property
    def course_input(self):
        return self.get_element(UploadLocators.COURSE_INPUT)

    @property
    def license_select(self):
        return Select(self.get_element(UploadLocators.LICENSE_SELECT))

    @property
    def rec_date_input(self):
        return self.get_element(UploadLocators.REC_DATE_INPUT)

    @property
    def start_hour_select(self):
        return Select(self.get_element(UploadLocators.START_HOUR_SELECT))

    @property
    def start_minute_select(self):
        return Select(self.get_element(UploadLocators.START_MINUTE_SELECT))

    @property
    def contributor_input(self):
        return self.get_element(UploadLocators.CONTRIBUTOR_INPUT)

    @property
    def type_input(self):
        return self.get_element(UploadLocators.TYPE_INPUT)

    @property
    def subject_input(self):
        return self.get_element(UploadLocators.SUBJECT_INPUT)

    @property
    def lang_input(self):
        return self.get_element(UploadLocators.LANG_INPUT)

    @property
    def desc_input(self):
        return self.get_element(UploadLocators.DESC_INPUT)

    @property
    def copyright_input(self):
        return self.get_element(UploadLocators.COPYRIGHT_INPUT)

    @property
    def single_file_radio(self):
        return self.get_element(UploadLocators.SINGLE_FILE_RADIO)

    @property
    def multi_file_radio(self):
        return self.get_element(UploadLocators.MULTI_FILE_RADIO)

    @property
    def single_file_local_radio(self):
        return self.get_element(UploadLocators.SINGLE_FILE_LOCAL_RADIO)

    @property
    def single_file_inbox_radio(self):
        return self.get_element(UploadLocators.SINGLE_FILE_INBOX_RADIO)

    @property
    def multi_file_presentation_local_radio(self):
        return self.get_element(UploadLocators.MULTI_FILE_PRESENTATION_LOCAL_RADIO)

    @property
    def multi_file_presenter_local_radio(self):
        return self.get_element(UploadLocators.MULTI_FILE_PRESENTER_LOCAL_RADIO)

    @property
    def multi_file_presentation_inbox_radio(self):
        return self.get_element(UploadLocators.MULTI_FILE_PRESENTATION_INBOX_RADIO)

    @property
    def multi_file_presenter_inbox_radio(self):
        return self.get_element(UploadLocators.MULTI_FILE_PRESENTER_INBOX_RADIO)

    @property
    def file_input_iframes(self):
        return self.get_elements(UploadLocators.FILE_INPUT_IFRAME)

    @property
    def local_file_selector(self):
        return self.get_element(UploadLocators.LOCAL_FILE_SELECTOR)

    @property
    def inbox_file_select(self):
        return Select(self.get_element(UploadLocators.INBOX_FILE_SELECT))

    @property
    def contains_slides_checkbox(self):
        return self.get_element(UploadLocators.CONTAINS_SLIDES_CHECKBOX)

    @property
    def workflow_select(self):
        return Select(self.get_element(UploadLocators.WORKFLOW_SELECT))

    @property
    def live_stream_checkbox(self):
        return self.get_element(UploadLocators.LIVE_STREAM_CHECKBOX)

    @property
    def multitrack_checkbox(self):
        return self.get_element(UploadLocators.MULTITRACK_CHECKBOX)

    @property
    def upload_button(self):
        return self.get_element(UploadLocators.UPLOAD_BUTTON)

    @property
    def upload_progress_dialog(self):
        return self.get_element(UploadLocators.UPLOAD_PROGRESS_DIALOG)

    def set_live_stream(self, enabled):
        self.set_checkbox(self.live_stream_checkbox, enabled)

    def set_multitrack(self, enabled):
        self.set_checkbox(self.multitrack_checkbox, enabled)

    def set_upload_files(self, presenter=None, presentation=None, combined=None,
                         is_inbox=False):
        """
        The MH upload UI does some crazy stuff with iframes.
        None of the iframe elements have unique ids, so this
        and the subsequent _private methods have to make some
        assumptions about the order in which they are returned
        by the locator elements.
        """
        if combined is not None:
            self.single_file_radio.click()
            if is_inbox:
                self.single_file_inbox_radio.click()
            else:
                self.single_file_local_radio.click()
            self.set_upload_file(self.file_input_iframes[0], combined, is_inbox)
        else:
            self.multi_file_radio.click()
            if is_inbox:
                self.multi_file_presentation_inbox_radio.click()
                self.multi_file_presenter_inbox_radio.click()
            else:
                self.multi_file_presentation_local_radio.click()
                self.multi_file_presenter_local_radio.click()
            self.set_upload_file(self.file_input_iframes[1], presentation, is_inbox)
            self.set_upload_file(self.file_input_iframes[2], presenter, is_inbox)

    def set_upload_file(self, iframe, file, is_inbox):
        self.driver.switch_to.frame(iframe)
        if is_inbox:
            self.inbox_file_select.select_by_value(file)
        else:
            # NOTE: this will silently fail if it's not an absolute path to an existing file
            self.local_file_selector.send_keys(abspath(file))
        self.driver.switch_to.default_content()

    def wait_for_upload_finish(self):
        WebDriverWait(self.driver, 1000000).until_not(visible(self.upload_progress_dialog))

class TrimPage(BasePage):

    @property
    def shortcut_table(self):
        return self.get_element(TrimLocators.SHORTCUT_TABLE)

    @property
    def trim_begin_input(self):
        return self.get_element(TrimLocators.CLIP_BEGIN_INPUT)

    @property
    def trim_end_input(self):
        return self.get_element(TrimLocators.CLIP_END_INPUT)

    @property
    def trim_ok_button(self):
        return self.get_element(TrimLocators.CLIP_OK_BUTTON)

    @property
    def split_remover(self):
        return self.get_element(TrimLocators.CLIP_REMOVE_BUTTON)

    @property
    def continue_button(self):
        return self.get_element(TrimLocators.CONTINUE_BUTTON)

    @property
    def clear_button(self):
        return self.get_element(TrimLocators.CLEAR_BUTTON)

    def trim(self):
        self.clear_button.click()
        sleep(1)
        # safe to assume media is > 10 seconds?
        self.shortcut_table.send_keys("l" * 20)
        sleep(1)
        self.shortcut_table.send_keys("v")
        sleep(1)
        self.shortcut_table.send_keys(Keys.ARROW_UP)
        sleep(1)
        self.shortcut_table.send_keys(Keys.DELETE)
        sleep(1)

        # media_length = timeparse(self.trim_end_input.get_attribute('value'))
        # trim_length = media_length / 10
        # self.trim_begin_input.clear()
        # self.trim_begin_input.send_keys(str(datetime.timedelta(seconds=trim_length)))
        # self.trim_ok_button.click()
        # sleep(2)
        # self.split_remover.click()

        self.continue_button.click()
        sleep(2)



import click
from unipath import Path
from fabric.api import env
from fabric.contrib.files import exists as remote_exists
from click.exceptions import UsageError
from functools import wraps
from urlparse import urljoin, urlsplit
from selenium import webdriver
from mh_selenium.pages import LoginPage

OPSWORKS_INBOX_PATH = '/var/matterhorn/inbox'
EC2_INBOX_PATH = '/home/data/opencast/inbox'

class ClickState(object):

    def __init__(self):
        self.username = None
        self.password = None
        self.base_url = None
        self.driver = None
        self.inbox_path = None
        self.host = None
        self.user = None

    @property
    def inbox(self):
        if self.inbox_path is not None:
            return self.inbox_path
        if remote_exists(OPSWORKS_INBOX_PATH):
            return OPSWORKS_INBOX_PATH
        elif remote_exists(EC2_INBOX_PATH):
            return EC2_INBOX_PATH
        else:
            raise UsageError("Can't determine remote inbox path")

    @property
    def inbox_dest(self):
        return Path(self.inbox).parent.child('files','collection','inbox')

pass_state = click.make_pass_decorator(ClickState, ensure=True)

def common_callback(ctx, option, value):
    state = ctx.ensure_object(ClickState)
    setattr(state, option.name, value)
    return value

def password_option(f):
    return click.option('-p','--password',
                        expose_value=False,
                        prompt=True,
                        help='MH admin login password',
                        callback=common_callback)(f)

def username_option(f):
    return click.option('-u','--username',
                        expose_value=False,
                        prompt=True,
                        help='MH admin login username',
                        callback=common_callback)(f)

def base_url_arg(f):
    return click.argument('base_url',
                          expose_value=False,
                          callback=common_callback)(f)

def user_option(f):
    return click.option('-u','--user',
                        expose_value=False,
                        help='The user to execute remote tasks as',
                        callback=common_callback)(f)

def host_option(f):
    return click.option('-H','--host',
                        expose_value=False,
                        help='host/ip of remote admin node',
                        callback=common_callback)(f)

def inbox_path_option(f):
    return click.option('-i', '--inbox_path',
                        expose_value=False,
                        help='alternate path to recording inbox',
                        callback=common_callback)(f)

def selenium_options(f):
    f = password_option(f)
    f = username_option(f)
    f = base_url_arg(f)
    return f

def inbox_options(f):
    f = host_option(f)
    f = user_option(f)
    f = inbox_path_option(f)
    return f

def init_fabric(click_cmd):
    @wraps(click_cmd)
    def wrapped(state, *args, **kwargs):

        # set up the fabric env
        env.host_string = state.host
        if state.user is not None:
            env.user = state.user

        return click_cmd(state, *args, **kwargs)
    return wrapped

def init_driver(init_path=''):
    def decorator(click_cmd):
        @wraps(click_cmd)
        def _wrapped_cmd(state, *args, **kwargs):

            state.driver = webdriver.Firefox()
            state.driver.implicitly_wait(10)
            state.driver.get(urljoin(state.base_url, init_path))

            if 'Login' in state.driver.title:
                page = LoginPage(state.driver)
                page.login(state.username, state.password)

            result = click_cmd(state, *args, **kwargs)

            state.driver.close()
            state.driver.quit()

            return result

        return _wrapped_cmd
    return decorator






import os
import re
import codecs
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

def read(path):
    return codecs.open(os.path.join(here, path), 'r', 'utf-8').read()

readme = read('README.md')
history = read('HISTORY.md')
version_file = read('mh_selenium/__init__.py')
version = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M).group(1)

install_requires = [
    "selenium==2.47.1",
    "click==5.1",
    "fabric==1.10.2",
    "pytimeparse==1.1.5",
    "unipath==1.1",
]

setup(
    name='dce-mh-selenium',
    version=version,
    packages=find_packages(),
    url='https://github.com/harvard-dce/mh-selenium',
    license='Apache 2.0',
    author='Jay Luker',
    author_email='jay_luker@harvard.edu',
    description='Selenium tasks and page objects for DCE Matterhorn',
    long_description=readme + '\n\n' + history,
    install_requires=install_requires,
    py_modules=["mh_driver", "mh_selenium"],
    entry_points='''
        [console_scripts]
        mh_driver=mh_driver:cli
    '''
)
